#ifndef _HAVE_UTIL_HEADER
#define _HAVE_UTIL_HEADER

char *
util_mprintf(const char *format, ...);

int 
util_ascii_to_hex(int c);

void 
util_dehttpize(char *z);

char *
util_extract_token(char *input, char **leftOver);

char *
util_htmlize(const char *in, int n);

char * 
util_httpize(const char* url);

int 
validDate(char *input);

int
parseIpv4Address(uint32_t *addr, const char *addrStr);

char*
code(char *key, char *str);

char*
decode(char *key, char *str);

#endif
